// A06_move
// General Move routines written
// Binary sketch size: 3,624 bytes (of a 14,336 byte maximum)

/* 
moves end with both feet flat on the ground
moves start at paceat and end at paceto, paceat updated to paceto
for walking and turning   paceto == paceC +- paceby
         [x] are proposed character commands - not yet implemented

  Ht() - [H] Halt with both feet together
  D()  - [d(*0.1secs)] Delay in 1/10 seconds
  Fd() - [F] Forward one pace
  Bk() - [B] Backward one pace 
  Rt() - [R] Right turn one pace
  Lt() - [L] Left turn one pace
  in turns it may be necessary to step one pace so the feet 
  are in the right relation to each other for the turn
  when going forward the steps are forward
  when going backward the steps are backward

  sp(sSpeed)      - [sp(sSpeed)] sets servo Speed, 0 - 255, default 2 
                               sets both rollspeed and pacespeed to sSpeed
  Spmax(speedmax) - [sm(speedmax)] sets speedmax, maximum speed for su sd, 0 - 255, default advised 15
  sc(sby)         - [sc(sby)] sets speed change for su sd, default 1
  su()            - [su] speed Up, increment sSpeed every call
  sd()            - [sd] speed Down, decrement sSpeed every call
  s0()            - [s0] detach servos from servo object - stops pulsing the servos
  s1()            - [s1] re-attach servos to servo object - starts pulsing the servos

  Hr() - Right Foot step forward to centre
  Hl() - Left Foot step forward to centre
  Fr() - Forward with Right foot one pace
  Fl() - Forward with Left foot one pace
  Br() - Backward with Right foot one pace
  Bl() - Backward with Left  foot one pace

  rC() - roll to Roll Centre, ie upright
  rR() - roll to Right
  rL() - roll to Left
  
  Sliding does not lift the feet
  pC() - slide to Pace Centre, ie feet side by side
  Lf() - slide left foot Forward  w.r.t. right
  Rb() - slide left foot Forward  w.r.t. right
  Lb() - slide left foot Backward w.r.t. right
  Rf() - slide left foot Backward w.r.t. right
*/
//===========================================================                               
// edit to select defaults for robot type

/* Bambino
// keep servos min and max to 50deg either side of mid=90deg for a nominal range of 100 
  const byte Drollmin  = 60;
  const byte DrollC    = 90;
  const byte Drollmax  =120;
  const byte Dpacemin  = 75;
  const byte DpaceC    = 90;
  const byte Dpacemax  =105;
  const byte Drollby   = 15;   // Default pace amount about rollC for walking
  const byte Dpaceby   = 15;   // Default pace amount about paceC for walking
  const byte DsSpeed   =  2;   // Default sSpeed for servos
  const byte Dsby      =  1;   // Default servo sSpeed increment for su sd
  const byte Dspeedmax =  4;   // Default maximum speed for servos
  const byte sframe    = 20;   // Servo frame rate, same as Servo.h frame rate, no point in it being less
  const byte servoRpin =7;     //Roll servo on pin 7   
  const byte servoPpin =8;     //Pace servo on pin 8   
// HobbyKing HK15128 servo
// +ve roll right, +ve left foot forward
*/  
// Jet
// keep servos min and max to 50deg either side of mid=90deg for a nominal range of 100 
  const byte Drollmin  = 60;
  const byte DrollC    = 90;
  const byte Drollmax  =120;
  const byte Dpacemin  = 75;
  const byte DpaceC    = 90;
  const byte Dpacemax  =105;
  const byte Drollby   = 25;   // Default pace amount about rollC for walking
  const byte Dpaceby   = 40;   // Default pace amount about paceC for walking
  const byte DsSpeed   =  2;   // Default sSpeed for servos
  const byte Dsby      =  1;   // Default servo sSpeed increment for su sd
  const byte Dspeedmax =  4;   // Default maximum speed for servos
  const byte sframe    = 20;   // Servo frame rate, same as Servo.h frame rate, no point in it being less
  const byte servoRpin =7;     //Roll servo on pin 7   
  const byte servoPpin =8;     //Pace servo on pin 8   
// Corona CS-538BP servo
// +ve roll left, +ve right foot forward
//
//===========================================================                               
#include <Servo.h>
Servo servoR;    // create servo object to control Roll servo 
Servo servoP;    // create servo object to control Pace servo 
/* Servo servoname - create servo object to control servo servoname  
   attach(pin) - Attaches a servo to an i/o pin.
   attach(pin,min,max) - Attaches to a pin setting min and max values in microseconds, 
                         default min is 544, max is 2400  
   write()     - Sets the servo angle in degrees.  
                 (invalid angle that is valid as pulse in microseconds is treated as microseconds)
   writeMicroseconds() - Sets the servo pulse width in microseconds 
   read()      - Gets the last written servo pulse width as an angle between 0 and 180. 
   readMicroseconds()  - Gets the last written servo pulse width in microseconds. (was read_us() in first release)
   attached()  - Returns true if there is a servo attached. 
   detach()    - Stops an attached i/o pin from pulsing its servo. 
 */ 
//===========================================================                               
#include <EEPROM.h>     // to read and write internal EEPROM
                        // EEPROM.write(address,bytevalue)
                        // bytevalue =EEPROM.read(address)
//----------------------------------------------------------- 
//EEPROM addresses
const byte Ef_walking =0;  // EEPROM address for f_walking boolean flag  
const byte Erollmin   =1;
const byte ErollC     =2;
const byte Erollmax   =3;
const byte Epacemin   =4;
const byte EpaceC     =5;
const byte Epacemax   =6;
const byte Eroll      =7;
const byte Epace      =8;

//===========================================================                               
byte rollC   =EEPROM.read(ErollC);
byte paceC   =EEPROM.read(EpaceC);
byte rollmin =EEPROM.read(Erollmin);
byte rollmax =EEPROM.read(Erollmax);
byte pacemin =EEPROM.read(Epacemin);
byte pacemax =EEPROM.read(Epacemax);
byte rollby  =EEPROM.read(Eroll);   // read default roll about rollC
byte paceby  =EEPROM.read(Epace);   // read default pace about paceC
byte rollat ;   // variable to store the roll servo position 
byte paceat ;   // variable to store the pace servo position 
byte rollto;   // desired roll-servo position 
byte paceto;   // desired pace-servo position 
boolean f_walking =true; // walk/stop
boolean f_Fd      =true; // false if last direction was backwards, true if forward 
byte Wrollby   =Drollby;  // roll amount for walking
byte Wpaceby   =Dpaceby;  // pace amount for walking
byte Trollby   =Drollby;  // roll amount for Turning
byte Tpaceby   =Dpaceby;  // pace amount for Turning
byte sSpeed    =DsSpeed;  // servo speed for both servos
byte rollspeed =DsSpeed;  // roll servo speed  
byte pacespeed =DsSpeed;  // pace servo speed
byte speedmax  =Dspeedmax;// max speed for auto-increment/decrement
byte sby       =Dsby;     // speed change for auto-increment/decrement
//byte cmnd ;    // current command
//===========================================================
void setup()
{ 
  f_walking =EEPROM.read(Ef_walking);   // read old state 
  f_walking =!f_walking ;			    // !true ->false, !false ->true
  EEPROM.write(Ef_walking,f_walking);   // save new state
  if (f_walking)  
    {  
    if (rollC==255)   {rollC =DrollC;}      // in case it is not initialised
    if (rollmin==255) {rollmin =Drollmin;}  // in case it is not initialised
    if (rollmax==255) {rollmax =Drollmax;}  // in case it is not initialised
    if (paceC==255)   {paceC =DpaceC;}      // in case it is not initialised
    if (pacemin==255) {pacemin =Dpacemin;}  // in case it is not initialised
    if (pacemax==255) {pacemax =Dpacemax;}  // in case it is not initialised
    if (rollby==255)  {rollby =Drollby;}    // in case it is not initialised
    if (paceby==255)  {paceby =Dpaceby;}    // in case it is not initialised 	
	
    rollat =rollC;                 // initialise rollat
    paceat =paceC;                 // initialise paceat    
    servoR.write(rollC);           // tell servo to go to position rollC 
    servoP.write(paceC);           // tell servo to go to position paceC
    servoR.attach(servoRpin);      // attaches servoRpin to the servoR object 
    servoP.attach(servoPpin);      // attaches servoPpin to the servoP object 
    }	
  //Serial.begin(9600);
} 
//-----------------------------------------------------------  
void loop()
{
  Fd();Fd();Fd();Fd();Rt();Rt();Bk();Bk();Bk();Bk();Lt();Lt();
}
//===========================================================
void D(byte param)  { delay(param *100); } 	                     // Delay 1/10 seconds
void Ht()           { if (paceat <paceC) {Hr();} else {Hl();} }  // Halt with both feet together
void Hr()           { rL(); pC(); rC(); }                        // Right Foot step forward to centre
void Hl()           { rR(); pC(); rC(); }                        // Left Foot step forward to centre
void Fd()           { if (paceat >paceC) {Fr();} else {Fl();} }  // step forward one pace
void Fr()           { rL(); Rf(); rC(); }                        // Right Foot step forward one pace
void Fl()           { rR(); Lf(); rC(); }                        // Left Foot step forward one pace
void Bk()           { if (paceat >paceC) {Bl();} else {Br();} }  // step backward one pace
void Br()           { rL(); Rb(); rC(); }                        // Right Foot step backward one pace
void Bl()           { rR(); Lb(); rC(); }                        // Left Foot step backward one pace
void Lt()  { if (f_Fd) { if (paceat >paceC) {Fr();}; Lf(); }     // Left turn  
             else      { if (paceat >paceC) {Bl();}; Lf(); } }    
void Rt()  { if (f_Fd) { if (paceat <paceC) {Fl();}; Rf(); }     // Right turn 
             else      { if (paceat <paceC) {Br();}; Rf(); } } 
    
//----------------------------------------------------------- 
void sp(int param)    { sSpeed =constrain(param,0,255); }     // set servo speed              // keep it in a byte
void Spmax(int param) { speedmax =constrain(param,0,255); }   // set maximum speed for su()   // keep it in a byte
void sc(int param)    { sby  =constrain(param,0,255); }       // set speed change             // keep it in a byte
void su()                                                     // increment speed <=speedmax
  { int temp =sSpeed +sby;  sSpeed =constrain(temp,1,speedmax);
    rollspeed =sSpeed;  pacespeed =sSpeed; }
void sd()                                                     // decrement speed >0
  { int temp =sSpeed -sby;  sSpeed =constrain(temp,1,speedmax);
    rollspeed =sSpeed;  pacespeed =sSpeed; }
void s0()  { servoR.detach();  servoP.detach(); }             // detach servos so stop pulsing them     
void s1()                                                     // re-attach servos
  { if (!servoR.attached()) {servoR.attach(servoRpin); }
    if (!servoP.attached()) {servoP.attach(servoPpin); } }
	
//----------------------------------------------------------- 
void Rf() { paceto =paceC -paceby; pacemove(); }   // slide Rightfoot Forward  w.r.t. left  
void Lb() { paceto =paceC -paceby; pacemove(); }   // slide Rightfoot Forward  w.r.t. left
void Lf() { paceto =paceC +paceby; pacemove(); }   // slide Leftfoot Forward w.r.t. right
void Rb() { paceto =paceC +paceby; pacemove(); }   // slide Leftfoot Forward w.r.t. right
void pC() { paceto =paceC; pacemove(); }           // Pace Centre, move feet so they are side by side
void rC() { rollto =rollC; rollmove(); }           // roll to Centre
void rR() { rollto =rollC +rollby; rollmove(); }   // roll to Right
void rL() { rollto =rollC -rollby; rollmove(); }   // roll to Left
  
//----------------------------------------------------------- 
void rollmove()    // do roll move  
  { if (rollspeed==0) {rollspeed =1;}     // dont hang if 0
    if (rollto > rollat)
      { for (rollat =rollat; rollat <rollto; rollat +=rollspeed)  {servoR.write(rollat); delay(sframe); } }  
     else     
      { for (rollat =rollat; rollat >rollto; rollat -=rollspeed)  {servoR.write(rollat); delay(sframe); } }
    rollat =rollto; servoR.write(rollat); delay(sframe);              // make sure servo is at rollto
  }

void pacemove()    // do pace move
  { if (pacespeed==0) {pacespeed =1;}     // dont hang if 0 
    if (paceto > paceat)
      { for (paceat =paceat; paceat <paceto; paceat +=pacespeed)  {servoP.write(paceat); delay(sframe); } }  
     else     
      { for (paceat =paceat; paceat >paceto; paceat -=pacespeed)  {servoP.write(paceat); delay(sframe); } }
    paceat =paceto; servoP.write(paceat); delay(sframe);              // make sure servo is at paceto
  }
//===========================================================


