// Jet_08_command
// added command interpreter for commands in strings
// Binary sketch size: 3,388 bytes (of a 14,336 byte maximum)

// moves end with both feet flat on the ground
// moves start at paceat and end at paceto, paceat updated to paceto
// for walking and turning paceto == paceC +- paceby
// [x] are the character commands

// Ht() - [H] Halt with both feet together
// D()  - [d(*0.1secs)] Delay in 1/10 seconds
// Fd() - [F] Forward one pace
// Bk() - [B] Backward one pace
// Rt() - [R] Right turn one pace
// Lt() - [L] Left turn one pace
// in turns it may be necessary to step one pace so the feet 
// are in the right relation to each other for the turn
// when going forward the steps are forward
// when going backward the steps are backward

// sp(sSpeed)      - [sp(sSpeed)] sets servo Speed, 0 - 255, default 2 
//                                sets both rollspeed and pacespeed to sSpeed
// Spmax(speedmax) - [sm(speedmax)] sets speedmax, maximum speed for su sd, 0 - 255, default advised 15
// sc(sby)         - [sc(sby)] sets speed change for su sd, default 1
// su()            - [su] speed Up, increment sSpeed every call
// sd()            - [sd] speed Down, decrement sSpeed every call
// s0()            - [s0] detach servos from servo object - stops pulsing the servos
// s1()            - [s1] re-attach servos to servo object - starts pulsing the servos

// Hr() - Right Foot step forward to centre
// Hl() - Left Foot step forward to centre
// Fr() - Forward with Right foot one pace
// Fl() - Forward with Left foot one pace
// Br() - Backward with Right foot one pace
// Bl() - Backward with Left  foot one pace

// rC() - roll to Roll Centre, ie upright
// rR() - roll to Right
// rL() - roll to Left
//Pacing does not lift the feet
// pC() - pace to Pace Centre, ie feet side by side
// pF() - pace Forward,  move right foot Forward  w.r.t. left
// pB() - pace Backward, move right foot Backward w.r.t. left

//===========================================================                               
// edit this moves string
char Act0[] ="FF R R L L BB H D50";    // spaces are ignored

//===========================================================                               
// Jet
// ===
// Corona CS-538BP servo
// +ve roll left, +ve right foot forward
// keep servos min and max to 50deg either side of mid=90deg for a nominal range of 100
//-----------------------------------------------------------                                
// edit this to your setup
// =======================
const byte servoRpin =6;      //Roll servo on pin 6   
const byte servoPpin =7;      //Pace servo on pin 7   
//===========================================================            
// edit these if you wish
// ======================
const byte sframe    =20; // Servo frame rate, same as Servo.h frame rate, no point in it being less
const byte Drollby   =25; // Default roll amount for walking
const byte Dpaceby   =40; // Default pace amount for walking
const byte Dspeedmax =15; // Default maximum speed for servos
const byte DsSpeed   = 2; // Default sSpeed for servos
const byte Dsby      = 1; // Default servo sSpeed increment for su sd
//===========================================================                               
#include <Servo.h>
Servo servoR;    // create servo object to control Roll servo 
Servo servoP;    // create servo object to control Pace servo 
/* Servo servoname - create servo object to control servo servoname  
   attach(pin) - Attaches a servo to an i/o pin.
   attach(pin,min,max) - Attaches to a pin setting min and max values in microseconds, 
                         default min is 544, max is 2400  
   write()     - Sets the servo angle in degrees.  
                 (invalid angle that is valid as pulse in microseconds is treated as microseconds)
   writeMicroseconds() - Sets the servo pulse width in microseconds 
   read()      - Gets the last written servo pulse width as an angle between 0 and 180. 
   readMicroseconds()  - Gets the last written servo pulse width in microseconds. (was read_us() in first release)
   attached()  - Returns true if there is a servo attached. 
   detach()    - Stops an attached i/o pin from pulsing its servo. 
 */ 
//----------------------------------------------------------- 
#include <EEPROM.h>     // to read and write internal EEPROM
                        // EEPROM.write(address,bytevalue)
                        // bytevalue =EEPROM.read(address)
//----------------------------------------------------------- 
//EEPROM addresses
const byte Ef_walking =0;  // EEPROM address for f_walking boolean flag
const byte ErollC =1;
const byte EpaceC =2;
//-----------------------------------------------------------                                
byte rollC;    // roll centre = 90 => upright
byte paceC;    // pace centre = 90 => feet together
byte rollat;   // current roll-servo position 
byte paceat;   // current pace-servo position 
byte rollto;   // desired roll-servo position 
byte paceto;   // desired pace-servo position 
boolean f_walking =true; // walk/stop
boolean f_Fd      =true; // false if last direction was backwards, true if forward 
byte Wrollby   =Drollby;  // roll amount for walking
byte Wpaceby   =Dpaceby;  // pace amount for walking
byte Trollby   =Drollby;  // roll amount for Turning
byte Tpaceby   =Dpaceby;  // pace amount for Turning
byte rollby    =Wrollby;  // roll amount for walking 
byte paceby    =Wpaceby;  // pace amount for walking 
byte sSpeed    =DsSpeed;  // servo speed for both servos
byte rollspeed =DsSpeed;  // roll servo speed  
byte pacespeed =DsSpeed;  // pace servo speed
byte speedmax  =Dspeedmax;// max speed for auto-increment/decrement
byte sby       =Dsby;     // speed change for auto-increment/decrement
byte p_Act0;    // pointer to next action in Act0
byte action;    // current action   
byte cmnd ;     // current command
//===========================================================
void setup()
{ 
  f_walking =EEPROM.read(Ef_walking);   // read old state 
  f_walking =!f_walking ;			    // !true ->false, !false ->true
  EEPROM.write(Ef_walking,f_walking);   // save new state
  if (f_walking)  
    {  
    rollC =EEPROM.read(ErollC);    // initialise rollC from current EEPROM ErollC
    if (rollC==255) {rollC =90;}   // in case it is not initialised
    paceC =EEPROM.read(EpaceC);    // initialise paceC from current EEPROM EpaceC
    if (paceC==255) {paceC =90;}   // in case it is not initialised
    rollat =rollC;                 // initialise rollat
    paceat =paceC;                 // initialise paceat    
    servoR.write(rollC);           // tell servo to go to position rollC 
    servoP.write(paceC);           // tell servo to go to position paceC
    servoR.attach(servoRpin);      // attaches servoRpin to the servoR object 
    servoP.attach(servoPpin);      // attaches servoPpin to the servoP object 
    }	
  //Serial.begin(9600);
} 
//-----------------------------------------------------------  
void loop()
{
  p_Act0 =0;
  do 					
    { cmnd =Act0[p_Act0];  action ++;            //ReadAction
      switch (cmnd)
      { case 'D':                                // delay  
          break;
      }    
    } while (cmnd !=0);                        // until end of string
}

//  Fd();Fd();Rt();Rt();Bk();Bk();Lt();Lt();

//===========================================================
void D(byte param)  // Delay 1/10 seconds
  { delay(param *100); } 	
void Ht()  // Halt with both feet together
  { if (paceat <paceC) {Hr();} else {Hl();} }
void Hr()  // Right Foot step forward to centre
  { rL(); pC(); rC(); }  
void Hl()  // Left Foot step forward to centre
  { rR(); pC(); rC(); }    

void Fd()  // step forward one pace
  { if (paceat <paceC) {Fr();} else {Fl();} }
void Fr()  // Right Foot step forward one pace
  { rL(); pF(); rC(); }  
void Fl()  // Left Foot step forward one pace
  { rR(); pB(); rC(); }  

void Bk()  // step backward one pace
  { if (paceat <paceC) {Bl();} else {Br();} }
void Br()  // Right Foot step backward one pace
  { rL(); pB(); rC(); }  
void Bl()  // Left Foot step backward one pace
  { rR(); pF(); rC(); }  
 
void Rt()  // Right turn
  { if (f_Fd) { if (paceat >paceC) {Fl();}; pF(); }
    else      { if (paceat >paceC) {Br();}; pF(); } }

void Lt()  // Left turn 
  { if (f_Fd) { if (paceat <paceC) {Fr();}; pB(); }
    else      { if (paceat <paceC) {Bl();}; pB(); } }
    
//----------------------------------------------------------- 
void sp(int param)    { sSpeed =constrain(param,0,255); }     // set servo speed              // keep it in a byte
void Spmax(int param) { speedmax =constrain(param,0,255); }   // set maximum speed for su()   // keep it in a byte
void sc(int param)    { sby  =constrain(param,0,255); }       // set speed change             // keep it in a byte
void su()                                                     // increment speed <=speedmax
  { int temp =sSpeed +sby;  sSpeed =constrain(temp,1,speedmax);  
    rollspeed =sSpeed;  pacespeed =sSpeed; }
void sd()                                                     // decrement speed >0
  { int temp =sSpeed -sby;  sSpeed =constrain(temp,1,speedmax);
    rollspeed =sSpeed;  pacespeed =sSpeed; }
void s0()  { servoR.detach();  servoP.detach(); }             // detach servos so stop pulsing them     
void s1()                                                     // re-attach servos
  { if (!servoR.attached()) {servoR.attach(servoRpin); }
    if (!servoP.attached()) {servoP.attach(servoPpin); } }
	
//----------------------------------------------------------- 
void Rf() { paceto =paceC -paceby; pacemove(); }   // slide Rightfoot Forward  w.r.t. left  
void Lb() { paceto =paceC -paceby; pacemove(); }   // slide Rightfoot Forward  w.r.t. left
void Lf() { paceto =paceC +paceby; pacemove(); }   // slide Leftfoot Forward w.r.t. right
void Rb() { paceto =paceC +paceby; pacemove(); }   // slide Leftfoot Forward w.r.t. right
void pC() { paceto =paceC; pacemove(); }           // Pace Centre, slide feet so they are side by side
void rC() { rollto =rollC; rollmove(); }           // roll to Centre
void rR() { rollto =rollC +rollby; rollmove(); }   // roll to Right
void rL() { rollto =rollC -rollby; rollmove(); }   // roll to Left   
  
//----------------------------------------------------------- 
void rollmove()    // do roll move  
  { if (rollspeed==0) {rollspeed =1;}     // dont hang if 0
    if (rollto > rollat)
      { for (rollat =rollat; rollat <rollto; rollat +=rollspeed)  {servoR.write(rollat); delay(sframe); } }  
     else     
      { for (rollat =rollat; rollat >rollto; rollat -=rollspeed)  {servoR.write(rollat); delay(sframe); } }
    rollat =rollto; servoR.write(rollat); delay(sframe);              // make sure servo is at rollto
  }

void pacemove()    // do pace move
  { if (pacespeed==0) {pacespeed =1;}     // dont hang if 0 
    if (paceto > paceat)
      { for (paceat =paceat; paceat <paceto; paceat +=pacespeed)  {servoP.write(paceat); delay(sframe); } }  
     else     
      { for (paceat =paceat; paceat >paceto; paceat -=pacespeed)  {servoP.write(paceat); delay(sframe); } }
    paceat =paceto; servoP.write(paceat); delay(sframe);              // make sure servo is at paceto
  }
//===========================================================

